@@ -68,11 +68,12 @@ func TestReadWriteJSONL(t *testing.T) {
 		"skip_both.jsonl",
 		"skip_both.jsonl",
 		"outdated.jsonl",
+		"known_outdated.jsonl",
 	}
 
 	for _, file := range files {
 		t.Run(file, func(t *testing.T) {
-			f := filepath.Join("testdata", "actions", file)
+			f := filepath.Join("testdata", "jsonl", file)
 			stdout := &bytes.Buffer{}
 			stderr := &bytes.Buffer{}
 
@@ -88,8 +89,8 @@ func TestReadWriteJSONL(t *testing.T) {
 			want := string(b)
 			have := stdout.String()
 
-			if want != have {
-				t.Fatalf("read content and output content differ\n%s", cmp.Diff(want, have))
+			if diff := cmp.Diff(want, have); diff != "" {
+				t.Fatalf("read content and output content differ\n%s", diff)
 			}
 		})
 	}
@@ -126,7 +127,7 @@ func TestWriteGoToStdout(t *testing.T) {
 		t.Run(tc.in, func(t *testing.T) {
 			stdout := &bytes.Buffer{}
 			stderr := &bytes.Buffer{}
-			status := newGen(stdout, stderr, io.Discard).run([]string{"test", "-s", filepath.Join("testdata", "actions", tc.in)})
+			status := newGen(stdout, stderr, io.Discard).run([]string{"test", "-s", filepath.Join("testdata", "jsonl", tc.in)})
 			if status != 0 {
 				t.Fatalf("exit status is non-zero: %d: %s", status, stderr.Bytes())
 			}
@@ -138,15 +139,15 @@ func TestWriteGoToStdout(t *testing.T) {
 			want := string(b)
 			have := stdout.String()
 
-			if want != have {
-				t.Fatalf("read content and output content differ\n%s", cmp.Diff(want, have))
+			if diff := cmp.Diff(want, have); diff != "" {
+				t.Fatalf("read content and output content differ\n%s", diff)
 			}
 		})
 	}
 }
 
 func TestWriteJSONLFile(t *testing.T) {
-	in := filepath.Join("testdata", "actions", "no_new_version.jsonl")
+	in := filepath.Join("testdata", "jsonl", "no_new_version.jsonl")
 	b, err := os.ReadFile(in)
 	if err != nil {
 		panic(err)
@@ -169,13 +170,13 @@ func TestWriteJSONLFile(t *testing.T) {
 	}
 	have := string(b)
 
-	if want != have {
-		t.Fatalf("read content and output content differ\n%s", cmp.Diff(want, have))
+	if diff := cmp.Diff(want, have); diff != "" {
+		t.Fatalf("read content and output content differ\n%s", diff)
 	}
 }
 
 func TestWriteGoFile(t *testing.T) {
-	in := filepath.Join("testdata", "actions", "no_new_version.jsonl")
+	in := filepath.Join("testdata", "jsonl", "no_new_version.jsonl")
 	out := filepath.Join("testdata", "go", "out.go")
 	defer os.Remove(out)
 
@@ -198,8 +199,8 @@ func TestWriteGoFile(t *testing.T) {
 	}
 	have := string(b)
 
-	if want != have {
-		t.Fatalf("read content and output content differ\n%s", cmp.Diff(want, have))
+	if diff := cmp.Diff(want, have); diff != "" {
+		t.Fatalf("read content and output content differ\n%s", diff)
 	}
 }
 
@@ -230,8 +231,8 @@ func TestFetchRemoteYAML(t *testing.T) {
 			want := string(b)
 			have := stdout.String()
 
-			if !cmp.Equal(want, have) {
-				t.Fatalf("fetched JSONL data does not match: %s", cmp.Diff(want, have))
+			if diff := cmp.Diff(want, have); diff != "" {
+				t.Fatalf("fetched JSONL data does not match: %s", diff)
 			}
 		})
 	}
@@ -246,18 +247,18 @@ func TestWriteOutdatedActionAsJSONL(t *testing.T) {
 		t.Fatal("exit status is non-zero:", status)
 	}
 
-	b, err := os.ReadFile(filepath.Join("testdata", "actions", "outdated.jsonl"))
+	b, err := os.ReadFile(filepath.Join("testdata", "jsonl", "outdated.jsonl"))
 	if err != nil {
 		panic(err)
 	}
 	want, have := string(b), stdout.String()
-	if want != have {
-		t.Fatal(cmp.Diff(want, have))
+	if diff := cmp.Diff(want, have); diff != "" {
+		t.Fatal(diff)
 	}
 }
 
 func TestLogOutput(t *testing.T) {
-	f := filepath.Join("testdata", "actions", "no_new_version.jsonl")
+	f := filepath.Join("testdata", "jsonl", "no_new_version.jsonl")
 	stdout := &bytes.Buffer{}
 	logged := &bytes.Buffer{}
 	status := newGen(stdout, io.Discard, logged).run([]string{"test", "-s", f, "-f", "jsonl"})
@@ -355,7 +356,7 @@ func TestCouldNotReadJSONLFile(t *testing.T) {
 	}
 	for _, tc := range testCases {
 		t.Run(tc.file, func(t *testing.T) {
-			f := filepath.Join("testdata", "actions", tc.file)
+			f := filepath.Join("testdata", "jsonl", tc.file)
 			stdout := io.Discard
 			stderr := &bytes.Buffer{}
 
@@ -373,7 +374,7 @@ func TestCouldNotReadJSONLFile(t *testing.T) {
 }
 
 func TestCouldNotCreateOutputFile(t *testing.T) {
-	f := filepath.Join("testdata", "actions", "no_new_version.jsonl")
+	f := filepath.Join("testdata", "jsonl", "no_new_version.jsonl")
 	out := filepath.Join("testdata", "this-dir-does-not-exit", "foo.jsonl")
 	stdout := io.Discard
 	stderr := &bytes.Buffer{}
@@ -398,7 +399,7 @@ func (w testErrorWriter) Write(b []byte) (int, error) {
 func TestWriteError(t *testing.T) {
 	for _, format := range []string{"go", "jsonl"} {
 		t.Run(format, func(t *testing.T) {
-			f := filepath.Join("testdata", "actions", "no_new_version.jsonl")
+			f := filepath.Join("testdata", "jsonl", "no_new_version.jsonl")
 			stdout := testErrorWriter{}
 			stderr := &bytes.Buffer{}
 
